#include "LED.h"
#include "Key.h"
#include "App.h"
#include "cw32r031_gtim.h"
#include "cw32r031_dma.h"
#include "cw32r031_uart.h"
#include "stdio.h"
#include "BIOS.h"
#include "pan1026_user.h"
#include "system_cw32r031.h"
#include "PAN1026.h"
#include "cw32r031_gpio.h"
#include "cw32r031_flash.h"
#include "cw32r031_gtim.h"
#define UART_BOUNDRATE 115200

uint8_t Work_State;
uint8_t ucTest_Mode;

/**
 * @brief  ʱӣͨPLLƵ64MHz
 * @param  None
 * @retval None
 */
void RCC_Configuration(void)
{
  __RCC_FLASH_CLK_ENABLE();
  FLASH_SetLatency(FLASH_Latency_2);    //Ƶʴ24MҪFlashWait=2

  RCC_HSI_Enable(RCC_HSIOSC_DIV1);
	
  RCC_HCLKPRS_Config(RCC_HCLK_DIV1);
  RCC_PCLKPRS_Config(RCC_PCLK_DIV1);
	
	RCC_SystemCoreClockUpdate(48000000);
}

/**
 * @brief  GPIOʼ
 * @param  None
 * @retval None
 */
void GPIO_Configuration(void)
{
  __RCC_GPIOA_CLK_ENABLE();
  __RCC_GPIOB_CLK_ENABLE();
  __RCC_GPIOC_CLK_ENABLE();
  __RCC_GPIOF_CLK_ENABLE();

  // PA15žͽģʽ
  GPIO_InitTypeDef GPIO_InitStructure = {0};
  GPIO_InitStructure.Pins = GPIO_PIN_15;
  GPIO_InitStructure.IT = GPIO_IT_NONE;
  GPIO_InitStructure.Mode = GPIO_MODE_INPUT;
  GPIO_Init(CW_GPIOA, &GPIO_InitStructure);
  /* TX/RX PIN input pulling high */

  PA08_AFx_UART1TXD();
  GPIO_InitStructure.Pins = GPIO_PIN_8;
  GPIO_InitStructure.IT = GPIO_IT_NONE;
  GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_Init(CW_GPIOA, &GPIO_InitStructure);
}

/**
 * @brief  GTIM1ʼ
 * @param  None
 * @retval None
 */
void GTIM_Init(void)
{

  GTIM_InitTypeDef GTIM_InitStruct;

  __RCC_GTIM1_CLK_ENABLE();

  GTIM_InitStruct.Mode = GTIM_MODE_TIME;
  GTIM_InitStruct.OneShotMode = GTIM_COUNT_CONTINUE;
  GTIM_InitStruct.Prescaler = GTIM_PRESCALER_DIV64; // DCLK = PCLK / 64 = 48MHz/64 = 0.75MHz
  GTIM_InitStruct.ReloadValue = 4000;
  GTIM_InitStruct.ToggleOutState = DISABLE;

  GTIM_TimeBaseInit(CW_GTIM1, &GTIM_InitStruct);

  GTIM_ITConfig(CW_GTIM1, GTIM_IT_OV, ENABLE);

  NVIC_EnableIRQ(GTIM1_IRQn);
  GTIM_Cmd(CW_GTIM1, ENABLE);
}

/**
 * @brief  ʱⲿж
 * @param  None
 * @retval None
 */
void Sys_Start(void)
{
  GTIM_Cmd(CW_GTIM1, ENABLE);
}

/**
 * @brief  UART1ʼ,Debugʹã115200
 * @param  None
 * @retval None
 */
void UART_Configuration(void)
{
  __RCC_UART1_CLK_ENABLE();
  USART_InitTypeDef USART_InitStructure;

  USART_InitStructure.USART_BaudRate = UART_BOUNDRATE;
  USART_InitStructure.USART_Over = USART_Over_16;
  USART_InitStructure.USART_Source = USART_Source_PCLK;
  USART_InitStructure.USART_UclkFreq = 48000000;
  USART_InitStructure.USART_StartBit = USART_StartBit_FE;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Tx;
  USART_Init(CW_UART1, &USART_InitStructure);
  USART_DirectionModeCmd(CW_UART1, USART_Mode_Tx, ENABLE);
}

/**
 * @brief  RF״̬ʼͨ(PB4)жRFTXRX MODE
 *        ͨϵʱްѡXN297L/NORDIC/BLE/CARRIER MODE
 * @param  None
 * @retval None
 */
void Status_Init(void)
{

  if (Key_Rd() == 0x01) /* ʼʱKEY1زģʽ */
  {
    ucTest_Mode = TEST_CARRIER_MODE;
  }
  else if (Key_Rd() == 0x04) /* ʼʱKEY3BLEģʽ */
  {
    ucTest_Mode = TEST_BLE_MODE;
  }
  else
  {
    ucTest_Mode = TEST_DEFAULT_MODE;
  }

  if (TRX_PIN_MODE)
  {
    Work_State = FUNCTION_RX;
    ucRFAPI_STATE = RFAPI_RX;
    pan1026_rx_mode();
#ifdef DEBUG_EN
    printf("Initial RX!\n");
#endif
    LED_ChangeFont(CHANNEL_TABLE[Channel_Index], LED_CHANNEL_MODE);
    DISP_Status = LED_CHANNEL_MODE;
  }
  else
  {
    Work_State = FUNCTION_TX;
    ucRFAPI_STATE = RFAPI_TX;
    ucTXMode = MODE_TX_SINGLE;
    pan1026_tx_mode();
#ifdef DEBUG_EN
    printf("Initial TX!\n");
#endif
    LED_ChangeFont(ucTXMode, LED_FUNCTION_MODE);
    DISP_Status = LED_FUNCTION_MODE;
  }
}

/**
 * @brief  ӳʼ-- KEYLEDTIMERUART, RF
 * @param  None
 * @retval None
 */
void BIOS_Init(void)
{
  __disable_irq();
  RCC_Configuration();
  GPIO_Configuration();
#ifdef DEBUG_EN
  UART_Configuration();
#endif
  Key_Init();
  LED_Init();
  GTIM_Init();
  __enable_irq();

  delay1ms(100);

  /* RFʼ */
  pan1026_user_init();
  Status_Init();

  /* ʱ */
  Sys_Start();
}
