#include "LED.h"
#include "App.h"
#include "stdio.h"

static uint8_t ucLED_DIS[4] = {EIGHT, EIGHT, EIGHT, EIGHT};
bool bLED_Enable = TRUE;

/* LED character Table */
const uint8_t LED_TAB[] =
    {
        0xC0, //'0'
        0xF9, //'1'
        0xA4, //'2'
        0xB0, //'3'
        0x99, //'4'
        0x92, //'5'
        0x82, //'6'
        0xF8, //'7'
        0x80, //'8'
        0x90, //'9'
        0xBF, //'-'
        0xFF, //' '
        0x86, //'E'
        0x88, //'R'
        0x46, //'c.'
        0x00, //'8.'
        0x8E, //'F'
        0x88, //'A'
        0x83, //'b'
        0xc6, //'C'
        0x0B  //'h'
};

/**
  * @brief  LED初始化,Initial LED Enable pin and CH455 pin
  * @param  None  
  * @retval None
  */
void LED_Init(void)
{
    uint8_t i;
    for (i = 0; i < sizeof(ucLED_DIS); i++)
    {
        ucLED_DIS[i] = 8;
    }
    GPIO_InitTypeDef GPIO_InitStructure = {0};

    GPIO_InitStructure.Pins = GPIO_PIN_11 | GPIO_PIN_12;
    GPIO_InitStructure.IT = GPIO_IT_NONE;
    GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_Init(CW_GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.Pins = GPIO_PIN_6 | GPIO_PIN_7;
    GPIO_InitStructure.IT = GPIO_IT_NONE;
    GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_Init(CW_GPIOB, &GPIO_InitStructure);

    PA11_SETHIGH();
    PA12_SETHIGH();

    LED_write_Byte(0x48, 0x81);
}

void start_up_sign(void)
{
    SCK_HIGH;
    SDA_LOW;
    delay10us(1);
}

void Ch455_writebyte(uint8_t ucdata)
{
    uint8_t i;
    for (i = 0; i < 8; i++)
    {
        SCK_LOW;
        if (ucdata & 0x80)
        {
            SDA_HIGH;
        }
        else
        {
            SDA_LOW;
        }
        ucdata <<= 1;
        SCK_HIGH;
        delay10us(1);
    }
}

void LED_ACK(void)
{
    SDA_LOW;
    SCK_LOW;
    SDA_HIGH;
    SCK_HIGH;
    delay10us(1);
    SCK_LOW;
    SDA_LOW;
}

void stop_sign(void)
{
    SCK_HIGH;
    delay10us(1);
    SDA_HIGH;
}

void LED_write_Byte(uint8_t data1, uint8_t data2)
{
    start_up_sign();
    Ch455_writebyte(data1);
    LED_ACK();
    Ch455_writebyte(data2);
    LED_ACK();
    stop_sign();
}

/**
  * @brief  LED显示
  * @param  None  
  * @retval None
  */
void LED_Display(void)
{
    static uint8_t ucLedDis_Bit_Index = 0;

    if (bLED_Enable) /* if Enable LED display */
    {
        if (ucLedDis_Bit_Index == 0)
        {
            LED_write_Byte(0x68, LED_TAB[ucLED_DIS[ucLedDis_Bit_Index] & 0x1F]);
            ucLedDis_Bit_Index = 1;
        }
        else if (ucLedDis_Bit_Index == 1)
        {
            LED_write_Byte(0x6A, LED_TAB[ucLED_DIS[ucLedDis_Bit_Index] & 0x1F]);
            ucLedDis_Bit_Index = 2;
        }
        else if (ucLedDis_Bit_Index == 2)
        {
            LED_write_Byte(0x6C, LED_TAB[ucLED_DIS[ucLedDis_Bit_Index] & 0x1F]);
            ucLedDis_Bit_Index = 3;
        }
        else if (ucLedDis_Bit_Index == 3)
        {
            LED_write_Byte(0x6E, LED_TAB[ucLED_DIS[ucLedDis_Bit_Index] & 0x1F]);
            ucLedDis_Bit_Index = 0;
        }
        else
        {
            ucLedDis_Bit_Index = 0;
        }
    }
    else /* if not Enable LED */
    {
        ucLedDis_Bit_Index = 0;
    }
}

/**
  * @brief  改变LED显示模式和显示数据
  * @param  usLED_Data：显示数据， ucLED_Mode显示模式
  * @retval None
  */
void LED_ChangeFont(uint16_t usLED_Data, uint8_t ucLED_Mode)
{
    /* CARRIER MODE */
    if (ucTest_Mode == TEST_CARRIER_MODE)
    {
        ucLED_DIS[3] = LED_C;
        ucLED_DIS[2] = LED_h;
        ucLED_DIS[1] = usLED_Data % 100 / 10;
        ucLED_DIS[0] = usLED_Data % 10;
    }
    /* RX FUNCTION */
    else if (Work_State == FUNCTION_RX)
    {
        switch (ucLED_Mode)
        {
        case LED_CHANNEL_MODE:
            ucLED_DIS[3] = LED_F;
            ucLED_DIS[2] = ucTest_Mode;
            ucLED_DIS[1] = usLED_Data % 100 / 10;
            ucLED_DIS[0] = usLED_Data % 10;
            break;
        case LED_FUNCTION_MODE:
            ucLED_DIS[3] = LED_F;
            ucLED_DIS[2] = ucTest_Mode;
            ucLED_DIS[1] = LED_NULL;
            ucLED_DIS[0] = usLED_Data % 10;
            break;
        case LED_CNT_MODE: /* CNT mode:(0000 - 9999) */
            ucLED_DIS[3] = usLED_Data % 10000 / 1000;
            ucLED_DIS[2] = usLED_Data % 1000 / 100;
            ucLED_DIS[1] = usLED_Data % 100 / 10;
            ucLED_DIS[0] = usLED_Data % 10;
            break;
        default:
            break;
        }
    }
    else if (Work_State == FUNCTION_TX)
    {
        /* TX FUNCTION */
        switch (ucLED_Mode)
        {
        case LED_CHANNEL_MODE:
            ucLED_DIS[3] = LED_F;
            ucLED_DIS[2] = ucTest_Mode;
            ucLED_DIS[1] = usLED_Data % 100 / 10;
            ucLED_DIS[0] = usLED_Data % 10;
            break;
        case LED_FUNCTION_MODE:
            ucLED_DIS[3] = LED_F;
            ucLED_DIS[2] = ucTest_Mode;
            ucLED_DIS[1] = SUB;
            if (usLED_Data == 1)
            {
                ucLED_DIS[0] = LED_A;
            }
            else if (usLED_Data == 2)
            {
                ucLED_DIS[0] = LED_b;
            }
            else if (usLED_Data == 3)
            {
                ucLED_DIS[0] = LED_C;
            }
            break;
        case LED_CNT_MODE:                  /* CNT mode */
            if (ucTXMode == MODE_TX_SINGLE) /* Single tx:(A000 - A999)*/
            {
                ucLED_DIS[3] = LED_A;
            }
            else
            {
                ucLED_DIS[3] = usLED_Data % 10000 / 1000; /* else:(0000-9999) */
            }
            ucLED_DIS[2] = usLED_Data % 1000 / 100;
            ucLED_DIS[1] = usLED_Data % 100 / 10;
            ucLED_DIS[0] = usLED_Data % 10;
            break;
        default:
            break;
        }
    }
}
