#include "hal.h"
#include "cw32r031_spi.h"
#include "cw32r031_gpio.h"
#include "cw32r031_rcc.h"
#include "delay.h"

#define SPI_SCK_LOW() PA02_SETLOW()
#define SPI_SCK_HIGH() PA02_SETHIGH()
#define SPI_MOSI_LOW() PA01_SETLOW()
#define SPI_MOSI_HIGH() PA01_SETHIGH()

#define SPI_SET_MOSI_IN() PA01_DIR_INPUT()
#define SPI_SET_MOSI_OUT() PA01_DIR_OUTPUT()
#define SPI_MOSI_READ() (CW_GPIOA->IDR & (uint8_t)GPIO_PIN_1)

#define SOFT_SPI_DELAY() delay10us(1)

/**
 * @brief  ӲSPIĳʼ, PAN1026ʹ
 * @param  None
 * @retval None
 */
void spi_for_rf_init(void)
{

    GPIO_InitTypeDef GPIO_InitStruct;
    GPIO_InitStruct.IT = GPIO_IT_NONE;
    GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_InitStruct.Pins = GPIO_PIN_3;
    GPIO_Init(CW_GPIOA, &GPIO_InitStruct);
    PA03_SETHIGH();

    SPI_InitTypeDef SPI_InitStructure;

    GPIO_InitStruct.IT = GPIO_IT_NONE;
    GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_InitStruct.Pins = GPIO_PIN_1 | GPIO_PIN_2;
    GPIO_Init(CW_GPIOA, &GPIO_InitStruct);

    GPIO_InitStruct.IT = GPIO_IT_NONE;
    GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
    GPIO_InitStruct.Pins = GPIO_PIN_0;
    GPIO_Init(CW_GPIOA, &GPIO_InitStruct);

//    PA00_AFx_SPI2MISO();
//    PA01_AFx_SPI2MOSI();
//    PA02_AFx_SPI2SCK();
//    PA03_AFx_SPI2CS();

    RCC_APBPeriphClk_Enable1(RCC_APB2_PERIPH_SPI1, ENABLE);

    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex; // ˫ȫ˫
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;                      // ģʽ
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;                  // ֡ݳΪ8bit
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;                         // ʱӿеƽΪ
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;                       // ڶز
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;                          // ƬѡźSSIĴ
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8; // ΪPCLK8Ƶ
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;                 // Чλ MSB շǰ
    SPI_InitStructure.SPI_Speed = SPI_Speed_Low;                       // SPI

    SPI_Init(CW_SPI1, &SPI_InitStructure);
    SPI_Cmd(CW_SPI1, ENABLE);
}

/**
 * @brief  ӲSPIPAN1026ַָָȵֽ
 * @param  addr ַ
 * @param  buf  Ҫ͵
 * @param  len  ݳ, ֽΪλ
 * @retval None
 */
void spi_for_rf_write_buf(uint8_t addr, uint8_t *buf, uint8_t len)
{
    CW_GPIOA->ODR &= (uint16_t)(~GPIO_PIN_3); /* CS Enable */

    while (SPI_GetFlagStatus(CW_SPI1, SPI_FLAG_TXE) == RESET)
        ;
    SPI_SendData(CW_SPI1, addr);

    while (SPI_GetFlagStatus(CW_SPI1, SPI_FLAG_RXNE) == RESET)
        ;
    SPI_ReceiveData(CW_SPI1);

    while (len--)
    {
        while (SPI_GetFlagStatus(CW_SPI1, SPI_FLAG_TXE) == RESET)
            ;
        SPI_SendData(CW_SPI1, *buf++);

        while (SPI_GetFlagStatus(CW_SPI1, SPI_FLAG_RXNE) == RESET)
            ;
        SPI_ReceiveData(CW_SPI1);
    }

    CW_GPIOA->ODR |= GPIO_PIN_3; /* CS Disable */
}

/**
 * @brief  ӲSPIPAN1026ַָȡָȵֽ
 * @param  addr ַ
 * @param  buf  ȡݻַ
 * @param  len  ݳ, ֽΪλ
 * @retval None
 */
void spi_for_rf_read_buf(uint8_t addr, uint8_t *buf, uint8_t len)
{
    CW_GPIOA->ODR &= (uint8_t)(~GPIO_PIN_3); /* CS Enable */

    while (SPI_GetFlagStatus(CW_SPI1, SPI_FLAG_TXE) == RESET)
        ;
    SPI_SendData(CW_SPI1, addr);

    while (SPI_GetFlagStatus(CW_SPI1, SPI_FLAG_RXNE) == RESET)
        ;
    SPI_ReceiveData(CW_SPI1);

    while (len--)
    {
        while (SPI_GetFlagStatus(CW_SPI1, SPI_FLAG_TXE) == RESET)
            ;
        SPI_SendData(CW_SPI1, 0);

        while (SPI_GetFlagStatus(CW_SPI1, SPI_FLAG_RXNE) == RESET)
            ;
        *buf++ = SPI_ReceiveData(CW_SPI1);
    }

    CW_GPIOA->ODR |= GPIO_PIN_3; /* CS Disable */
}

/**
 * @brief  IOģSPIĳʼ, PAN1026ʹ
 * @param  None
 * @retval None
 */
void soft_spi_for_rf_init(void)
{
    GPIO_InitTypeDef GPIO_InitStruct;
    GPIO_InitStruct.IT = GPIO_IT_NONE;
    GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_InitStruct.Pins = GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3;
    GPIO_Init(CW_GPIOA, &GPIO_InitStruct);
    PA03_SETHIGH();
    PA01_SETLOW();
    PA02_SETLOW();
}

/**
 * @brief  IOģSPI, дһֽ
 * @param  dat 
 * @retval None
 */
static uint8_t soft_spi_for_rf_write_byte(uint8_t dat)
{
    uint8_t cnt;

    SPI_SET_MOSI_OUT(); /* ÿдʱ, вҪ */

    for (cnt = 0; cnt < 8; cnt++)
    {
        SPI_SCK_LOW();
        SOFT_SPI_DELAY();

        /*  */
        if (dat & 0x80)
            SPI_MOSI_HIGH();
        else
            SPI_MOSI_LOW();

        dat <<= 1;
        SPI_SCK_HIGH();
        SOFT_SPI_DELAY();
    }

    SPI_SET_MOSI_IN(); /* Ҫ: һʱǰ, ͽл */

    SPI_SCK_LOW();
    SOFT_SPI_DELAY();

    return 0;
}

/**
 * @brief  IOģSPI, һֽ
 * @param  None
 * @retval 
 */
static uint8_t soft_spi_for_rf_read_byte(void)
{
    uint8_t cnt;
    uint8_t dat = 0;

    for (cnt = 0; cnt < 8; cnt++)
    {
        SPI_SCK_LOW();
        SOFT_SPI_DELAY();
        SPI_SCK_HIGH();
        SOFT_SPI_DELAY();

        /* ȡ, ע SPI_MOSI_READ() ͵ƽ0ߵƽز1 */
        dat <<= 1;
        if (0 != SPI_MOSI_READ())
        {
            dat |= 0x01;
        }
    }

    SPI_SCK_LOW();
    SOFT_SPI_DELAY();

    return dat;
}

/**
 * @brief  IOģSPI, PAN1026ַָдָȵֽ
 * @param  addr ַ
 * @param  buf  ȡݻַ
 * @param  len  ݳ, ֽΪλ
 * @retval None
 */
void soft_spi_for_rf_write_buf(uint8_t addr, uint8_t *buf, uint8_t len)
{
    GPIO_WritePin(CW_GPIOA, GPIO_PIN_3, GPIO_Pin_RESET); /* CS Enable */

    soft_spi_for_rf_write_byte(addr);
    while (len--)
    {
        soft_spi_for_rf_write_byte(*buf++);
    }

    GPIO_WritePin(CW_GPIOA, GPIO_PIN_3, GPIO_Pin_SET); /* CS Disable */
}

/**
 * @brief  IOģSPI, PAN1026ַָȡָȵֽ
 * @param  addr ַ
 * @param  buf  ȡݻַ
 * @param  len  ݳ, ֽΪλ
 * @retval None
 */
void soft_spi_for_rf_read_buf(uint8_t addr, uint8_t *buf, uint8_t len)
{
    GPIO_WritePin(CW_GPIOA, GPIO_PIN_3, GPIO_Pin_RESET); /* CS Enable */

    soft_spi_for_rf_write_byte(addr);

    while (len--)
    {
        *buf = soft_spi_for_rf_read_byte();
        buf++;
    }

    GPIO_WritePin(CW_GPIOA, GPIO_PIN_3, GPIO_Pin_SET); /* CS Disable */
}
