#include "PAN1026.h"
#include "config_rf.h"
#include "delay.h"

static pan1026_com_interface *com_interface;
#define READ_REG_BUF      com_interface->rf_read_buf
#define WRITE_REG_BUF     com_interface->rf_write_buf

/**
 * @brief  PAN1026ָĴһֽ
 * @param  reg
 * @retval ֵ
 */
static uint8_t read_reg_byte(uint8_t reg)
{
  uint8_t res;

  READ_REG_BUF(reg, &res, 1);
  return res;
}

/**
 * @brief  PAN1026ָĴдһֽ
 * @param  reg
 * @param  byte
 * @retval None
 */
static void write_reg_byte(uint8_t reg, uint8_t byte)
{
  WRITE_REG_BUF(reg, &byte, 1);
}

/**
 * @brief  PAN1026 1.8VּĴʼòͨҪ޸
 * @param  None
 * @retval 0: SPII2Cд  1SPII2Cд
 */
static uint8_t pan1026_1v8_reg_init(void)
{
  uint8_t dat[5];

  dat[0] = 0x20;
  /*  Ⱥ */
  /* dat[1] = 0xd0;
     dat[2] = 0xff; */
  /* С Ȳ1dB */
  dat[1] = 0xc0;
  dat[2] = 0xfb;
  WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_SETUP_RX, dat, 3);

  dat[0] = 0x48;
  dat[1] = 0x2a;
  dat[2] = 0xfd;
  WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_SETUP_RXTX, dat, 3);

  dat[0] = 0xfd;
  dat[1] = 0x3f;
  WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_SETUP_RF, dat, 2);

  dat[0] = 0xab;
#if ((CONFIG_CHIP_VERSION == PAN1026_VERSION_A) || (CONFIG_CHIP_VERSION == PAN1026_VERSION_D))
  dat[1] = 0xc9;
#endif
#if (CONFIG_CHIP_VERSION == PAN1026_VERSION_B)
  dat[1] = 0x09;
#endif
  dat[2] = 0x5f;
  WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_SETUP_PLL, dat, 3);

  /*  */
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_DEMOD_CAL, 0x1a | CONFIG_SCR);

  dat[0] = 0x60;
  dat[1] = 0x0e;
  WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_RF_CAL, dat, 2);

  write_reg_byte(PAN1026_ACTIVATE, 0x73);

  /* Feature, ֱ64ֽģʽ, ʹ32ֽģʽ */
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_FEATURE, IRQ_OUT_NORMAL | IRQ_PIN | IQ_SW | PAYLOAD_64 | CONFIG_DPL | CONFIG_ACK_PAY | CONFIG_DYN_ACK);

  /* CRC16TX MODE */
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_CONFIG, XN297L_MODE | ENABLE_RXIRQ_PIN | ENABLE_TXIRQ_PIN | ENABLE_MAX_RT_IRQ_PIN | CONFIG_CRC | TX_MODE);

  /* 5ֽڵַ*/
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_AW_RXADDR, AW_5BYTES | CONFIG_RX_PIPE);

  /* noIRQ_noWORKʹ */
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_DYNPD, ENABLE_VCOCAL_ONLY1 | NO_IRQ_NO_WORK | CONFIG_DPL_PIPE);

  /* ֤I2CSPIǷ(дκһR/WȨ޵ļĴ) */
  if (read_reg_byte(PAN1026_R_REGISTER + PAN1026_DYNPD) == (ENABLE_VCOCAL_ONLY1 | NO_IRQ_NO_WORK | CONFIG_DPL_PIPE))
    return 1;
  else
    return 0;
}

/**
 * @brief  PAN1026ʼ
 * @param  interface ӿ,  pan1026_com_interface 
 * @retval 0ʼʧ  1:ʼɹ
 */
uint8_t pan1026_init(pan1026_com_interface *interface)
{
  uint8_t reg0_3v_data;
  uint8_t reg1_3v_data_array[2];
  uint8_t trx_addr[CONFIG_ADDR_LEN] = CONFIG_ADDR;

  com_interface = interface;

  /* ͨŽӿ */
  reg1_3v_data_array[0] = 0x4f;
  if (CONFIG_DRIVE_MODE == PAN1026_SPI_MODE)
  {
    reg1_3v_data_array[1] = 0x08;
    reg0_3v_data = 0x00;
  }
  else if (CONFIG_DRIVE_MODE == PAN1026_I2C_MODE)
  {
    reg1_3v_data_array[1] = 0x0d;
    reg0_3v_data = 0x14;
  }
  else
  {
    reg1_3v_data_array[1] = 0x09;
    reg0_3v_data = 0x00;
  }
  /* Դϵ, Ĵλ */
  WRITE_REG_BUF(PAN1026_3V_REG1_WR, reg1_3v_data_array, 2);

  write_reg_byte(PAN1026_3V_REG0_WR, reg0_3v_data);
  /* ϵȶʱ2 */
  delay1ms(2);
  write_reg_byte(PAN1026_3V_REG0_WR, reg0_3v_data | 0x01);

  /* 1.8VĴʼ */
  if (!pan1026_1v8_reg_init())
    return 0;

  /*  */
  pan1026_set_mode(CONFIG_WORK_MODE);
  pan1026_set_channel(CONFIG_CHANNEL);
  pan1026_set_power(CONFIG_POWER);
  pan1026_set_rate(CONFIG_RATE);
  pan1026_set_rx_payload_len(CONFIG_PAYLOAD_WIDTH);
  pan1026_set_addr_width(CONFIG_ADDR_LEN);
  pan1026_set_rx_addr(trx_addr, CONFIG_ADDR_LEN);
  pan1026_set_tx_addr(trx_addr, CONFIG_ADDR_LEN);
  /* ΪTX MODERX MODE */
  if (CONFIG_TRX_MODE == PAN1026_TX_MODE)
  {
    pan1026_tx_mode();
  }
  else
  {
    pan1026_rx_mode();
  }
  pan1026_ce_enable();

#if (CONFIG_WORK_MODE == PAN1026_BLE_MODE)
  uint8_t filter[2] = CONFIG_BLE_FILTER_ADDR;
#if (CONFIG_BLE_FILTER == PAN1026_BLE_FILTER_2BYTE)
  pan1026_set_ble_whitelist(filter, 2, 1); /* ʹܹˣ2ֽ*/
#elif (CONFIG_BLE_FILTER == PAN1026_BLE_FILTER_1BYTE)
  pan1026_set_ble_whitelist(filter, 1, 1); /* ʹܹˣ1ֽ*/
#else
  pan1026_set_ble_whitelist(filter, 2, 0); /* */
#endif
#endif

  return 1;
}

/**
  * @brief  ùģʽ,
            ,ĬϿ
  * @param  mode Ҫõģʽ
  * PAN1026_BURST_MODE   XN297Lͨ
  * PAN1026_ENHANCE_MODE  XN297Lǿ
  * PAN1026_BLE_MODE      BLEģʽ
  * @retval None
  */
void pan1026_set_mode(uint8_t mode)
{
  uint8_t dat[5];
  uint8_t temp;

  dat[0] = 0xd3;
  dat[1] = 0x59;
  dat[2] = 0x0c;

  dat[3] = 0x50;
  dat[4] = 0x9c;
  WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_BB_CAL, dat, 5);

  /* ģʽ */
  temp = read_reg_byte(PAN1026_CONFIG);
  temp &= ~(0x03 << 6);
  temp |= (((uint8_t)mode & 0x30) << 2);
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_CONFIG, temp);

  /* ǿǿвͬ */
  temp = read_reg_byte(PAN1026_EN_AA) & 0xC0;
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_EN_AA, temp | CONFIG_AUTO_ACK_PIPE);
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_SETUP_RETR, CONFIG_AUTO_RETR);
}

/**
 * @brief  ˯, öʧ, ͨ³ʼ
 * @param  None
 * @retval None
 */
void pan1026_deep_sleep(void)
{
  pan1026_ce_disable();
  write_reg_byte(PAN1026_3V_REG1_WR, 0x40);
}

/**
 * @brief  Ƶ
 * @param  ch ʵƵ = ch + 2400MHz
 * @retval None
 */
void pan1026_set_channel(uint8_t ch)
{
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_RF_CH, ch + 64);
}

/**
 * @brief  CRC
 * @param  type ҪõCRC
 * PAN1026_CRC16         CRC16
 * PAN1026_CRC8          CRC8
 * PAN1026_CRC_DISABLE   رCRC
 * @retval None
 */
void pan1026_set_crc(uint8_t type)
{
  uint8_t temp;
  temp = read_reg_byte(PAN1026_CONFIG);
  temp &= 0xf9;
  temp |= (uint8_t)type;
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_CONFIG, temp);
}

/**
 * @brief  ÷书
 * @param  power ҪõĹ
 * PAN1026_POWER_P_12	  12dbm
 * PAN1026_POWER_P_8	  8dbm
 * PAN1026_POWER_P_6	  6dbm
 * PAN1026_POWER_P_3	  3dbm
 * PAN1026_POWER_P_2	  2dbm
 * PAN1026_POWER_P_1	  1dbm
 * PAN1026_POWER_P_0	  0dbm
 * PAN1026_POWER_N_8	  -8dbm
 * @retval None
 */
void pan1026_set_power(uint16_t power)
{
  uint8_t dat[3];

  dat[0] = 0x0;
  dat[1] = (uint8_t)(power >> 8);
  dat[2] = (uint8_t)(power);

#if (CONFIG_CHIP_VERSION == PAN1026_VERSION_D)
  dat[2] |= BIT5_MASK;
#endif

  WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_RF_CAL2, dat, 3);
}

/**
 * @brief  
 * @param  rate Ҫõ
 * PAN1026_RATE_1M   1Mbps
 * PAN1026_RATE_2M   2Mbps
 * @retval None
 */
void pan1026_set_rate(uint8_t rate)
{
  uint8_t dat[3];
  uint8_t demod_cal, en_aa_reg;
  demod_cal = read_reg_byte(PAN1026_DEMOD_CAL) & 0x01;
  en_aa_reg = read_reg_byte(PAN1026_EN_AA) & 0x3f;

  if (rate == PAN1026_RATE_1M)
  {
    en_aa_reg |= DR_1M;
    dat[0] = 0x48;
    dat[1] = 0x2a;
    dat[2] = 0xfd;
    demod_cal += 0x1a;
  }
  else if (rate == PAN1026_RATE_2M)
  {
    en_aa_reg |= DR_2M;
    dat[0] = 0x49;
    dat[1] = 0x2a;
    dat[2] = 0xfd;
    demod_cal += 0x18;
  }
  else
  {
  }
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_EN_AA, en_aa_reg);
  WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_SETUP_RXTX, dat, 3);
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_DEMOD_CAL, demod_cal);
}

/**
 * @brief  򿪻رNo AckACK Payload, ǿģʽʹ, ĬϹر
 * @param  enable 1  0ر
 * @retval None
 */
void pan1026_set_noack_ackpayload(uint8_t enable)
{
  uint8_t temp;
  temp = read_reg_byte(PAN1026_FEATURE);
  temp &= ~(ENABLE_ACK_PAY | ENABLE_DYN_ACK);
  if (enable == 1)
  {
    write_reg_byte(PAN1026_ACTIVATE, 0x73);
    temp |= (ENABLE_ACK_PAY | ENABLE_DYN_ACK);
  }
  else
  {
    write_reg_byte(PAN1026_DEACTIVATE, 0x8c);
    temp &= ~(ENABLE_ACK_PAY | ENABLE_DYN_ACK);
  }
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_FEATURE, temp);
}

/**
 * @brief  Txַ, BLEģʽ
 * @param  addr ҪõTxַ
 * @param  addr_len   ַ,
 * @retval None
 */
void pan1026_set_tx_addr(uint8_t *addr, uint8_t addr_len)
{
  WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_TX_ADDR, addr, addr_len);
}

/**
 * @brief  Rx(ͨ0)ַ, BLEģʽ
 * @param  addr ҪõRxַ
 * @param  addr_len  ַ
 * @retval None
 */
void pan1026_set_rx_addr(uint8_t *addr, uint8_t addr_len)
{
  WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_RX_ADDR_P0, addr, addr_len);
}

/**
 * @brief  Tx,Rxַ, BLEģʽ
 * @param  addr_width  ַ (3<=addr_width<=5)
 * @retval None
 */
void pan1026_set_addr_width(uint8_t addr_width)
{
  uint8_t temp;
  temp = read_reg_byte(PAN1026_AW_RXADDR) & 0x3f;
  temp |= (addr_width - 2) << 6;
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_AW_RXADDR, temp);
}

/**
 * @brief  BLE Access Address, BLEģʽʹ
 * @param  addr Ҫõĵַ, 4ֽ
 * @retval 0 óɹ, 1 ʧ, 2 ʧҳҪ³ʼ
 */
uint8_t pan1026_set_ble_access_addr(uint8_t *addr)
{
  write_reg_byte(PAN1026_W_PAGE_OF_1P8V_REG, 1); /* 1.8VĴҳ */
  if (1 == read_reg_byte(PAN1026_R_PAGE_OF_1P8V_REG))
  {
    WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_ACCESS_ADDRESS, addr, 4);

    write_reg_byte(PAN1026_W_PAGE_OF_1P8V_REG, 0);
    if (0 != read_reg_byte(PAN1026_R_PAGE_OF_1P8V_REG))
    {
      return 2;
    }
    return 0;
  }
  return 1;
}

/**
  * @brief  BLE, BLEģʽ״̬
  * @param  filter , ݰжӦλõݺͱһ²ϱ
  * @param  len    ݳ, ֧12ֽ;
                   Ϊ1ĿAdvData7ֽ(1ʼ), filterֽ֮Ӧ;
                   Ϊ2ĿAdvData6,7ֽ(1ʼ), filterֽڶӦ7
                   ֽ, 1ֽڶӦ6ֽ
  * @param  en     1, 0ذ; Ϊ0ʱfilterlen, ɴֵ
  * @retval 0 óɹ, 1 ʧ, 2 ʧҳҪ³ʼ
  */
uint8_t pan1026_set_ble_whitelist(uint8_t *filter, uint8_t len, uint8_t en)
{
  uint8_t dat[6] = {0};

  write_reg_byte(PAN1026_W_PAGE_OF_1P8V_REG, 1); /* 1.8VĴҳ */
  if (1 == read_reg_byte(PAN1026_R_PAGE_OF_1P8V_REG))
  {
    if (en == 1)
    {
      dat[5] = *filter;
      if (len == 2)
      {
        dat[4] = *(filter + 1);

        write_reg_byte(PAN1026_W_REGISTER + PAN1026_P1_REG_01, 2);
      }
      else
      {
        write_reg_byte(PAN1026_W_REGISTER + PAN1026_P1_REG_01, 1);
      }

      WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_P1_REG_00, dat, 6);
    }
    else
    {
      write_reg_byte(PAN1026_W_REGISTER + PAN1026_P1_REG_01, 0x00);
    }

    write_reg_byte(PAN1026_W_PAGE_OF_1P8V_REG, 0); /* ҳл */
    if (0 != read_reg_byte(PAN1026_R_PAGE_OF_1P8V_REG))
    {
      return 2;
    }
    return 0;
  }
  return 1;
}

/**
 * @brief  Tx״̬
 * @param  None
 * @retval None
 */
void pan1026_tx_mode(void)
{
  uint8_t temp;
  pan1026_ce_disable();

  temp = read_reg_byte(PAN1026_CONFIG);
  temp &= ~(0x01);
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_CONFIG, temp);

  pan1026_flush_tx_fifo();
  pan1026_flush_rx_fifo();
  pan1026_clear_status();

  pan1026_ce_enable();
}

/**
 * @brief  Rx״̬
 * @param  None
 * @retval None
 */
void pan1026_rx_mode(void)
{
  uint8_t temp;
  temp = read_reg_byte(PAN1026_CONFIG);
  temp |= 0x01;
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_CONFIG, temp);
}

/**
 * @brief  Rx Payload(ͨ0), BLEģʽ
 * @param  len Ҫõĳ, һµݰŻ
 *             297ģʽ    0 < len <= 64
 * @retval None
 */
void pan1026_set_rx_payload_len(uint8_t len)
{
  if (len <= 64)
  {
    write_reg_byte(PAN1026_W_REGISTER + PAN1026_RX_PW_P0, len);
  }
}

/**
 * @brief  λֽ
 * @param  buf 
 * @param  len ݳ
 * @retval None
 */
static void pan1026_byte_bit_decode(uint8_t *buf, uint8_t len)
{
  uint8_t data1;
  uint8_t data2;
  uint8_t i;
  uint8_t j;
  uint8_t *p1;
  uint8_t *p2;

  i = len >> 1;
  p1 = buf;
  p2 = buf + len - 1;

  if ((len & 1) != 0)
  {
    /* len, мһֽwhileﲻᴦ, ȶ⴦ */
    if (len > 1)
    {
      buf = buf + i;
    }
    data1 = *buf;

    *buf = 0;
    for (j = 0; j < 7; j++)
    {
      *buf += ((data1 >> j) & 0x01);
      *buf = *buf << 1;
    }
    *buf += (data1 >> 7);
  }

  while (i--)
  {
    data1 = *p1;
    data2 = *p2;
    *p1 = *p2 = 0;
    for (j = 0; j < 7; j++)
    {
      *p1 += ((data2 >> j) & 0x01);
      *p1 = *p1 << 1;

      *p2 += ((data1 >> j) & 0x01);
      *p2 = *p2 << 1;
    }
    *p1 += (data2 >> 7);
    *p2 += (data1 >> 7);
    p1++;
    p2--;
  }
}

/**
 * @brief  
 * @param  buf  
 * @param  len  ݳ
 * @retval None
 */
void pan1026_send(uint8_t *buf, uint8_t len)
{
  WRITE_REG_BUF(PAN1026_W_TX_PAYLOAD, buf, len);
}

/**
 * @brief  ACK Payload, ǿģʽ·Ͷ˼⵽pan1026_tx_rx_done־
 * @param  buf     ACK Payload
 * @param  maxsize ACK Payload󳤶, ȵݰᱻ
 * @retval ֵ0ʾPayload, Ϊ0ʾPayloadȳޱ
 */
uint8_t pan1026_read_ackpayload(uint8_t *buf, const uint8_t maxsize)
{
  uint8_t len;

  len = read_reg_byte(PAN1026_R_RX_PL_WID);
  if (len <= maxsize)
  {
    READ_REG_BUF(PAN1026_R_RX_PAYLOAD, buf, len);
    pan1026_byte_bit_decode(buf, len);
  }
  else
  {
    len = 0;
  }

  pan1026_flush_rx_fifo();
  pan1026_clear_status();

  return len;
}

/**
 * @brief  дACK Payload, ǿģʽ½ն˵
 * @param  buf ACK Payload
 * @param  len ACK Payload
 * @retval None
 */
void pan1026_write_ackpayload(uint8_t *buf, uint8_t len)
{
  WRITE_REG_BUF(PAN1026_W_ACK_PAYLOAD, buf, len);
}

/**
 * @brief  ȡյ, ն˼⵽ pan1026_rx_done Ǻ
 * @param  buf     ݻ
 * @param  maxsize 󳤶, ȵݰᱻ
 * @retval ֵ0ʾյݳ, Ϊ0ʾݰȳޱ
 */
uint8_t pan1026_receive(uint8_t *buf, const uint8_t maxsize)
{
  uint8_t len = 0;

  len = read_reg_byte(PAN1026_R_RX_PL_WID);
  if (len <= maxsize)
  {
    READ_REG_BUF(PAN1026_R_RX_PAYLOAD, buf, len);
    pan1026_byte_bit_decode(buf, len);
  }
  else
  {
    len = 0;
  }

  return len;
}

/**
 * @brief  CEʹ, ƵӲ
 * @param  None
 * @retval None
 */
void pan1026_ce_enable(void)
{
  write_reg_byte(PAN1026_CE_SPI_ON, 0x00);
}

/**
 * @brief  CEر, ƵӲͣ
 * @param  None
 * @retval None
 */
void pan1026_ce_disable(void)
{
  write_reg_byte(PAN1026_CE_SPI_OFF, 0x00);
}

/**
 * @brief  Tx FIFO
 * @param  None
 * @retval None
 */
void pan1026_flush_tx_fifo(void)
{
  write_reg_byte(PAN1026_FLUSH_TX, 0x00);
}

/**
 * @brief  Rx FIFO
 * @param  None
 * @retval None
 */
void pan1026_flush_rx_fifo(void)
{
  write_reg_byte(PAN1026_FLUSH_RX, 0x00);
}

/**
 * @brief  ȡTxRx״̬
 * @param  None
 * @retval ȡ״̬־
 */
uint8_t pan1026_get_status(void)
{
  return (read_reg_byte(PAN1026_STATUS) & 0x70);
}

/**
 * @brief  TxRx״̬, ״̬ʱ״̬ܼTxRx
 * @param  None
 * @retval None
 */
void pan1026_clear_status(void)
{
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_STATUS, 0x70);
}

/**
 * @brief  ز
 * @param  None
 * @retval None
 */
void pan1026_carrier(void)
{
  uint8_t dat[3] = {0x0b, 0xdf, 0x02};

  write_reg_byte(PAN1026_W_REGISTER + PAN1026_DYNPD, 0x40);
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_CONFIG, 0x06);
  write_reg_byte(PAN1026_W_REGISTER + PAN1026_DEMOD_CAL, 0xaf);
  WRITE_REG_BUF(PAN1026_W_REGISTER + PAN1026_DEM_CAL2, dat, 3);

  pan1026_ce_disable();
}
