#include "App.h"
#include "LED.h"
#include "key.h"
#include "stdio.h"
#include "pan1026.h"
#include "pan1026_user.h"
#include "config_rf.h"
#include "RFAPI.h"
#include "BIOS.h"
uint32_t Payload_Count = 0;
uint16_t tx_only_count = 0;
uint16_t time_out_count = 0;

bool bRFAPI_IRQ = FALSE;
bool flag_flush = TRUE;
enum RFAPI_CommuSate ucRFAPI_STATE;
RF_PAYLOAD RF_Payload;
uint8_t Channel_Index = 19; // 2478
uint8_t const CHANNEL_TABLE[] = {2, 6, 10, 14, 18, 22, 26, 30, 34, 38, 42, 46, 50, 54, 58, 62, 66, 70, 74, 78};
uint8_t const CHANNEL_TABLE_CARRIER[] = {2, 50, 78};

#if (EN_DYNPLOAD == 1)
uint8_t PayLoadLength = 0;
#endif

#if (EN_ACK_PAYLOAD == 1)
uint8_t PayLoadAck_Data[PAYLOAD_WIDTH] = {0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08};
#endif

/******************************************************************************/
//            RF_Channel_Next
//                Hop to next channel
/******************************************************************************/
void RFAPI_ChannelNext(void)
{

    if ((Channel_Index == 0) || (Channel_Index >= sizeof(CHANNEL_TABLE)))
    {
        Channel_Index = sizeof(CHANNEL_TABLE) - 1;
    }
    else
    {
        Channel_Index--;
    }

    pan1026_set_channel(CHANNEL_TABLE[Channel_Index]);
}

/******************************************************************************/
//            RF_PacketData
//                packet tx data
/******************************************************************************/
void RFAPI_PacketData(void)
{
    RF_Payload.ucPayload[0] = 0x12;
    RF_Payload.ucPayload[1] = 0x23;
    RF_Payload.ucPayload[2] = 0x34;
    RF_Payload.ucPayload[3] = 0x45;
    RF_Payload.ucPayload[4] = 0x56;
    RF_Payload.ucPayload[5] = 0x67;
    RF_Payload.ucPayload[6] = 0x78;
    RF_Payload.ucPayload[7] = 0x86;
}

/******************************************************************************/
//            RFAPI_TxState
//                In IRFAPI_TxState ,Data will be write into RF if TX enabled
/******************************************************************************/
void RFAPI_TxState(void)
{
    if (!(TX_RQ)) // if don't need to  transmit data
    {
        return;
    }
    TX_RQ = FALSE;

    RFAPI_PacketData();

    if (PAN1026_TX_DONE == pan1026_user_tx(RF_Payload.ucPayload, rev_data_length))
    {
        //  printf("tx ok \r\n");
    }
    else
    {
        // printf("tx err \r\n");
    }
}

static uint8_t rx_len;
static uint8_t rx_buffer[64];

static uint8_t rf_tx_buffer[39] =
    {
        0x42, 0xFF,                         // 0x42, 0xFFPDU Header   0xFFΪPDUЧ  Ҫݺ Manufacturer Specific Data޸
        0x66, 0x55, 0x44, 0x33, 0x22, 0x11, // 0x66-0x11:Adv Address û޸
        // ADV Data
        0x05, 0x09, 'T', 'E', 'M', 'P', // 	AD Structure 1:豸֣û޸ ֳȷ仯ע޸ BLE_FIX_FRAME_LENGTHֵ
        0x02, 0x01, 0x1A,               // 	AD Structure 2:Flags for discoverability. һ㲻޸
        0xFF, 0xFF, 0xF0, 0xFF          // 	AD Structure 3:Manufacturer Specific Data ûЭݷڸýṹ,һ0xFFΪֽ֡ Ҫʵ޸
};

void rf_tx_send()
{
    uint8_t tx_count = 3;
    uint8_t temp_length = rev_data_length;

    rf_tx_buffer[1] = 19 + temp_length;
    rf_tx_buffer[17] = 3 + temp_length;

    for (uint8_t i = 0; i < temp_length; i++)
    {
        rf_tx_buffer[21 + i] = rx_buffer[i];
    }

    while (tx_count > 0)
    {
        pan1026_set_channel(2);
        pan1026_user_tx(rf_tx_buffer, 21 + temp_length);
        pan1026_set_channel(26);
        pan1026_user_tx(rf_tx_buffer, 21 + temp_length);
        pan1026_set_channel(80);
        pan1026_user_tx(rf_tx_buffer, 21 + temp_length);
        tx_count--;
    }
}

void rf_tx(void)
{
    rf_tx_send();
}

/******************************************************************************/
//            BB_RX_State
//                In RX status ,Judge data has been received and read them
/******************************************************************************/
void RFAPI_RXState(bool RX_Continue)
{

    rx_len = pan1026_user_rx(rx_buffer, 64);
    if (rx_len > 0)
    {
        /* ӡյݳȺ */
        /*
          printf("rx len: %d\r\n", rx_len);
          printf("data: ");
          for (uint8_t i = 0; i < rx_len; i++)
          {
              printf("%02x ", buf[i]);
          }
        */

        if (Pan1026_RxMode == PAN1026_RX_BLE_MODE)
        {
            pan1026_set_mode(PAN1026_BLE_MODE);
            pan1026_tx_mode();
            pan1026_flush_tx_fifo();
            pan1026_clear_status();
            rf_tx();

            pan1026_set_mode(PAN1026_BURST_MODE);
            pan1026_set_channel(PAN1026_BLE_DEFAULT_CHANNEL);
            pan1026_rx_mode();
        }

        ucRFAPI_STATE = RFAPI_RX;
        RX_READY = TRUE;
    }
}

/******************************************************************************/
//            Test_Mode
//                Set RF into TestMode
/******************************************************************************/
void RFAPI_TestMode(void)
{

    if (ucTest_Mode == 0) // TEST_DEFAULT_MODE = 0, Ĭģʽ
    {

        return; // normal mode
    }
    else if (ucTest_Mode == TEST_CARRIER_MODE)
    {
        // RF_Carrier(DEFAULT_CHANNEL);
        pan1026_carrier();
        LED_ChangeFont(PAN1026_BURST_DEFAULT_CHANNEL, LED_FUNCTION_MODE);
#ifdef DEBUG_EN
        printf("Carrier mode!  \r\n");
#endif

        while (1)
        { /*
          Key_Scan();
         if(DATA_READY)                                                                //if Key Event
         {
           DATA_READY = FALSE;
          if(ucKey_RD == KEY3)                                                        //if press Key1 button down
             {
             if((Channel_Index == 0)||(Channel_Index >= sizeof(CHANNEL_TABLE_CARRIER)))
               {
               Channel_Index = sizeof(CHANNEL_TABLE_CARRIER)-1;
               }
              else
               {
                Channel_Index--;
                }

            //RF_Carrier( CHANNEL_TABLE_CARRIER[Channel_Index]);                                //set RF into carry mode
            LED_ChangeFont(CHANNEL_TABLE_CARRIER[Channel_Index],LED_FUNCTION_MODE);
             }

           }*/
            LED_Display();
        }
    }
    else
    {
        ucTest_Mode = 0; // normal mode
    }
}

/******************************************************************************/
//            BB_StateMachine
//                BB TX/RX Work StateMachine
/******************************************************************************/
void RFAPI_StateMachine(void)
{
    switch (ucRFAPI_STATE)
    {
    case RFAPI_TX: // RF in Idle
        RFAPI_TxState();
        break;
    case RFAPI_RX: // RF in RX
        RFAPI_RXState(TRUE);
        break;
    case RFAPI_HALT: // RF in shutdown
        break;
    default:
        break;
    }
}

//********************************end of file*********************************//
