#include "hal.h"
#include "cw32r031_gpio.h"
#include "cw32r031_rcc.h"
#include "system_cw32r031.h"


/* ģI2CSDA */
#define SOFT_I2C_SDA_IN() PA01_DIR_INPUT()
#define SOFT_I2C_SDA_OUT() PA01_DIR_OUTPUT()

/* ģI2CSCL SDA */
#define SOFT_I2C_SCL_HIGH() PA02_SETHIGH()
#define SOFT_I2C_SCL_LOW() PA02_SETLOW()
#define SOFT_I2C_SDA_HIGH() PA01_SETHIGH()
#define SOFT_I2C_SDA_LOW() PA01_SETLOW()

/* ģI2CĶSDA״̬ */
#define SOFT_I2C_READ_SDA() (CW_GPIOA->IDR & (uint8_t)GPIO_PIN_1)

/**
 * @brief  IOģI2Cʼ, PAN1026ʹ
 * @param  None
 * @retval None
 */
void soft_i2c_for_rf_init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitStruct.IT = GPIO_IT_NONE;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pins = GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3;
  GPIO_Init(CW_GPIOA, &GPIO_InitStruct);

	PA03_SETHIGH();
  //(CW_GPIOA->IDR & (uint8_t)GPIO_PIN_1)
}

/**
 * @brief  IOģI2Cʼź
 * @param  None
 * @retval None
 */
void soft_i2c_start(void)
{
  SOFT_I2C_SDA_OUT();

  SOFT_I2C_SDA_HIGH();
  SOFT_I2C_SCL_HIGH();
  

  SOFT_I2C_SDA_LOW();
  

  SOFT_I2C_SCL_LOW();
}

/**
 * @brief  IOģI2Cֹͣź
 * @param  None
 * @retval None
 */
void soft_i2c_stop(void)
{
  SOFT_I2C_SDA_OUT();

  SOFT_I2C_SCL_LOW();
  SOFT_I2C_SDA_LOW();
  

  SOFT_I2C_SCL_HIGH();
  SOFT_I2C_SDA_HIGH();
  
}

/**
 * @brief  IOģI2CȴACK
 * @param  None
 * @retval 0ɹ 1ʧ
 */
int soft_i2c_wait_ack(void)
{
  uint32_t ucErrTime = 0;

  SOFT_I2C_SDA_IN();

  

  SOFT_I2C_SCL_HIGH();
  
  while (0 != SOFT_I2C_READ_SDA())
  {
    ucErrTime++;
    if (ucErrTime > 500)
    {
      soft_i2c_stop();
      return 1;
    }
    
  }
  SOFT_I2C_SCL_LOW();

  return 0;
}

/**
 * @brief  IOģI2C ACK
 * @param  None
 * @retval None
 */
void soft_i2c_ack(void)
{
  SOFT_I2C_SCL_LOW();
  SOFT_I2C_SDA_OUT();
  SOFT_I2C_SDA_LOW();
  SOFT_I2C_SCL_HIGH();
  SOFT_I2C_SCL_LOW();
}

/**
 * @brief  IOģI2C NoACK
 * @param  None
 * @retval None
 */
void soft_i2c_no_ack(void)
{
  SOFT_I2C_SCL_LOW();
  SOFT_I2C_SDA_OUT();
  SOFT_I2C_SDA_HIGH();
  SOFT_I2C_SCL_HIGH();
  SOFT_I2C_SCL_LOW();
}

/**
 * @brief  IOģI2Cһֽ
 * @param  byte Ҫ͵ֽ
 * @retval None
 */
void soft_i2c_send_byte(uint8_t byte)
{
  int t;

  SOFT_I2C_SDA_OUT();
  SOFT_I2C_SCL_LOW();
  for (t = 0; t < 8; t++)
  {
    if ((byte & 0x80) != 0)
    {
      SOFT_I2C_SDA_HIGH();
    }
    else
    {
      SOFT_I2C_SDA_LOW();
    }
    byte <<= 1;

    
    SOFT_I2C_SCL_HIGH();
    
    SOFT_I2C_SCL_LOW();

    if (t == 7)
    {
      SOFT_I2C_SDA_LOW();
    }
    
  }
  SOFT_I2C_SCL_HIGH();
}

/**
 * @brief  IOģI2Cһֽ
 * @param  ack Ϊ1ACK Ϊ0NoACK
 * @retval None
 */
uint8_t soft_i2c_read_byte(int ack)
{
  uint8_t receive = 0;
  uint8_t i;

  SOFT_I2C_SDA_IN();

  for (i = 0; i < 8; i++)
  {
    SOFT_I2C_SCL_LOW();
    SOFT_I2C_SCL_HIGH();

    receive <<= 1;
    if (0 != SOFT_I2C_READ_SDA())
      receive++;
  }

  if (ack == 1)
    soft_i2c_ack();
  else
    soft_i2c_no_ack();

  return receive;
}

/**
 * @brief  IOģI2CPAN1026ַָдָֽ
 * @param  addr ַ
 * @param  buf  ȡݻַ
 * @param  len  ݳ, ֽΪλ
 * @retval None
 */
void soft_i2c_write_buf(uint8_t addr, uint8_t *buf, uint8_t len)
{
  soft_i2c_start();

  soft_i2c_send_byte(0xe2); /* ӻַ, PAN1026̶0xe2 */
  soft_i2c_wait_ack();
  soft_i2c_send_byte(addr);
  soft_i2c_wait_ack();

  while (len--)
  {
    soft_i2c_send_byte(*buf++);
    soft_i2c_wait_ack();
  }

  soft_i2c_stop();
}

/**
 * @brief  IOģI2CPAN1026ַָȡָֽ
 * @param  addr ַ
 * @param  buf  ȡݻַ
 * @param  len  ݳ, ֽΪλ
 * @retval None
 */
void soft_i2c_read_buf(uint8_t addr, uint8_t *buf, uint8_t len)
{
  int count = 0;

  soft_i2c_start();

  soft_i2c_send_byte(0xe2);
  soft_i2c_wait_ack();
  soft_i2c_send_byte(addr);
  soft_i2c_wait_ack();

  soft_i2c_start();
  soft_i2c_send_byte(0xe2 + 1); /* ַλ1ʾ */
  soft_i2c_wait_ack();

  for (count = 0; count < len; count++)
  {
    if (count != len - 1)
      buf[count] = soft_i2c_read_byte(1);
    else
      buf[count] = soft_i2c_read_byte(0); /* һֽNACK */
  }

  soft_i2c_stop();
}
