#include "pan1026_user.h"
#include "string.h"
#include "PAN1026.h"
#include "hal.h"
#include "config_rf.h"
#include "delay.h"
#include "cw32r031_gpio.h"

#define IRQ_STATUS GPIO_ReadPin(CW_GPIOF, GPIO_PIN_1)

static pan1026_com_interface interface;

#if (CONFIG_IRQ == PAN1026_IRQ_ENABLE)
/**
 * @brief  RF IRQڳʼ
 * @param  None
 * @retval None
 */
static void gpio_for_rf_irq_init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure = {0};
    GPIO_InitStructure.Pins = GPIO_PIN_1;
    GPIO_InitStructure.IT = GPIO_IT_FALLING | GPIO_IT_RISING;
    GPIO_InitStructure.Mode = GPIO_MODE_INPUT;
    GPIO_Init(CW_GPIOF, &GPIO_InitStructure);

    GPIOF_INTFLAG_CLR(bv1);
    NVIC_EnableIRQ(GPIOF_IRQn);
}
#endif

/**
 * @brief  ʼ
 * @param  None
 * @retval None
 */
void pan1026_user_init(void)
{
    /* RF IRQڳʼ */
#if (CONFIG_IRQ == PAN1026_IRQ_ENABLE)
    gpio_for_rf_irq_init();
#endif

    /* PAN1026ͨŽӿ, ʼ */
#if (CONFIG_DRIVE_MODE == PAN1026_I2C_MODE)
    soft_i2c_for_rf_init();
    interface.rf_write_buf = soft_i2c_write_buf;
    interface.rf_read_buf = soft_i2c_read_buf;
    pan1026_init(&interface);
#elif (CONFIG_DRIVE_MODE == PAN1026_SPI_MODE)
    spi_for_rf_init();
    interface.rf_write_buf = spi_for_rf_write_buf;
    interface.rf_read_buf = spi_for_rf_read_buf;
    pan1026_init(&interface);
#else
    soft_spi_for_rf_init();
    interface.rf_write_buf = soft_spi_for_rf_write_buf;
    interface.rf_read_buf = soft_spi_for_rf_read_buf;
    pan1026_init(&interface);
#endif
}

/**
 * @brief  ָ
 * @param  buf 
 * @param  len ݳ
 * @retval PAN1026_TX_DONEʾͳɹ
 */
uint8_t pan1026_user_tx(uint8_t *buf, uint8_t len)
{
    uint8_t sta;

    pan1026_send(buf, len);

    /*
     * PAN1026 QFNװ, ƼIRQ;
     *
     * IRQ, ͨ pan1026_get_status ѯ״̬ĴѯǷ,
     * TxʱдĴӰƵ, Լʱ, ʱʱҪڷǰ
     * ʱ
     */
#if (CONFIG_IRQ == PAN1026_IRQ_ENABLE)
    while (0 != IRQ_STATUS)
        ;
    sta = pan1026_get_status();
#else
    delay1ms(2);
    while ((sta = pan1026_get_status()) == PAN1026_STA_NONE)
        ;
#endif

    pan1026_flush_tx_fifo();
    pan1026_clear_status();

    return sta;
}

/**
 * @brief  ǿ(ACK PAYLOAD)ģʽ
 * @param  buf 
 * @param  buf_len ݳ
 * @param  payload ݻ
 * @param  payload_len ݳ
 * @retval յݳ, 0ʾδյ
 */
uint8_t pan1026_user_tx_enhance(uint8_t *buf, uint8_t buf_len, uint8_t *payload,
                                uint8_t payload_len)
{
    uint8_t sta;
    uint8_t len = 0;

    pan1026_send(buf, buf_len);

    /*
     * PAN1026 QFNװ, ƼIRQ;
     *
     * IRQ, ͨ pan1026_get_status ѯ״̬ĴѯǷ,
     * TxʱдĴӰƵ, Լʱ, ʱʱҪڷǰ
     * ʱ
     */
#if (CONFIG_IRQ == PAN1026_IRQ_ENABLE)
    while (0 != IRQ_STATUS)
        ;
    sta = pan1026_get_status();
#else
    delay1ms(2);
    while ((sta = pan1026_get_status()) == PAN1026_STA_NONE)
        ;
#endif

    if (sta == PAN1026_TRX_DONE)
    {
        len = pan1026_read_ackpayload(payload, payload_len);
    }

    pan1026_flush_tx_fifo();
    pan1026_clear_status();

    return len;
}

/**
 * @brief  
 * @param  buf      ݻ
 * @param  max_size 󳤶, ȵݰᱻ
 * @retval յݳ, 0ʾδյ
 */
uint8_t pan1026_user_rx(uint8_t *buf, uint8_t max_size)
{
    uint8_t sta;
    uint8_t len = 0;

    /*
     * PAN1026 QFNװ, ƼIRQ, Rx״̬ԼСѯĴ
     * ԽȵӰ
     */
#if (CONFIG_IRQ == PAN1026_IRQ_ENABLE)
    if (0 != IRQ_STATUS)
    {
        return 0;
    }
#endif

    sta = pan1026_get_status();
    if (sta != PAN1026_STA_NONE)
    {
        if (sta == PAN1026_RX_DONE)
        {
            len = pan1026_receive(buf, max_size);
        }

        pan1026_flush_rx_fifo();
        pan1026_clear_status();
    }

    return len;
}

/**
 * @brief  ǿͽ
 * @param  buf      ݻ
 * @param  max_size 󳤶, ȵݰᱻ
 * @retval յݳ
 */
uint8_t pan1026_user_rx_enhance(uint8_t *buf, uint8_t max_size)
{
    uint8_t sta;
    uint8_t len = 0;

    /*
     * PAN1026 QFNװ, ƼIRQ
     */
#if (CONFIG_IRQ == PAN1026_IRQ_ENABLE)
    if (0 != IRQ_STATUS)
    {
        return 0;
    }

    /*
     * ע:
     * ǿģʽյݺӲԶתTxACK, ӲACKʱдĴ
     * Ӱ췢;
     *
     * BB_CALĴ(ַ0x1f):
     * XN297Lģʽǿ, յݵʼACKʱԼ196us;
     * NORDICģʽǿԼ150us;
     *
     * ѯIRQʱ300usȷACKټ;
     *
     * ִʱSPII2Cʼհй, ʱС, 
     * ʱ, ҪȷִACKϵʱڲдĴ.
     */
    delay10us(30);
#endif

    sta = pan1026_get_status();
    if (sta != PAN1026_STA_NONE)
    {
        if (sta == PAN1026_RX_DONE)
        {
#if (CONFIG_IRQ == PAN1026_IRQ_DISABLE)
            delay10us(30);
#endif
            len = pan1026_receive(buf, max_size);
        }

        pan1026_flush_rx_fifo();
        pan1026_clear_status();
    }

    return len;
}

/**
 * @brief  ǿʹPayload, յݺ, һACKһͬ
 *         ظ
 * @param  buf     ݻ
 * @param  max_size 󳤶, ȵݰᱻ
 * @param  payload ظ
 * @param  len     ظݳ
 * @retval յݳ
 */
uint8_t pan1026_user_rx_enhance_ackpayload(uint8_t *buf, uint8_t max_size,
                                           uint8_t *payload, uint8_t payload_len)
{
    uint8_t sta;
    uint8_t len = 0;

    /*
     * PAN1026 QFNװ, ƼIRQ
     */
#if (CONFIG_IRQ == PAN1026_IRQ_ENABLE)
    if (0 != IRQ_STATUS)
    {
        return 0;
    }

    /*
     * жϲ, ӲԶӦǰдPayload;
     * ʱü, ȶȡյдpayload
     */
    pan1026_write_ackpayload(payload, payload_len);

    delay10us(30);
#endif

    sta = pan1026_get_status();
    if (sta != PAN1026_STA_NONE)
    {
        if (sta == PAN1026_RX_DONE)
        {
#if (CONFIG_IRQ == PAN1026_IRQ_DISABLE)
            pan1026_write_ackpayload(payload, payload_len);
            delay10us(30);
#endif
            len = pan1026_receive(buf, max_size);
        }

        pan1026_flush_rx_fifo();
        pan1026_clear_status();
    }

    return len;
}
